@@ -3,6 +3,8 @@
 namespace Icinga\Module\Director\Controllers;
 
 use gipfl\Web\Widget\Hint;
+use Icinga\Module\Director\Monitoring;
+use Icinga\Module\Director\Web\Table\ObjectsTableService;
 use ipl\Html\Html;
 use gipfl\IcingaWeb2\Link;
 use gipfl\IcingaWeb2\Url;
@@ -10,6 +12,7 @@ use gipfl\IcingaWeb2\Widget\Tabs;
 use Exception;
 use Icinga\Module\Director\CustomVariable\CustomVariableDictionary;
 use Icinga\Module\Director\Db\AppliedServiceSetLoader;
+use Icinga\Module\Director\DirectorObject\Lookup\ServiceFinder;
 use Icinga\Module\Director\Forms\IcingaAddServiceForm;
 use Icinga\Module\Director\Forms\IcingaServiceForm;
 use Icinga\Module\Director\Forms\IcingaServiceSetForm;
@@ -22,23 +25,65 @@ use Icinga\Module\Director\Web\Controller\ObjectController;
 use Icinga\Module\Director\Web\SelfService;
 use Icinga\Module\Director\Web\Table\IcingaHostAppliedForServiceTable;
 use Icinga\Module\Director\Web\Table\IcingaHostAppliedServicesTable;
-use Icinga\Module\Director\Web\Table\IcingaHostServiceTable;
 use Icinga\Module\Director\Web\Table\IcingaServiceSetServiceTable;
-use Icinga\Module\Director\Web\Widget\HostServiceRedirector;
 
 class HostController extends ObjectController
 {
     protected function checkDirectorPermissions()
     {
-        if (in_array($this->getRequest()->getActionName(), [
+        if ($this->isServiceAction() && (new Monitoring())->authCanEditService(
+            $this->Auth(),
+            $this->getParam('name'),
+            $this->getParam('service')
+        )) {
+            return;
+        }
+
+        if ($this->isServicesReadOnlyAction()) {
+            $this->assertPermission('director/monitoring/services-ro');
+            return;
+        }
+
+        if ($this->hasPermission('director/hosts')) { // faster
+            return;
+        }
+
+        if ($this->canModifyHostViaMonitoringPermissions($this->getParam('name'))) {
+            return;
+        }
+
+        $this->assertPermission('director/hosts'); // complain about default hosts permission
+    }
+
+    protected function isServicesReadOnlyAction()
+    {
+        return in_array($this->getRequest()->getActionName(), [
             'servicesro',
             'findservice',
-            'invalidservice'
-        ])) {
-            $this->assertPermission('director/monitoring/services-ro');
-        } else {
-            $this->assertPermission('director/hosts');
+            'invalidservice',
+        ]);
+    }
+
+    protected function isServiceAction()
+    {
+        return in_array($this->getRequest()->getActionName(), [
+            'servicesro',
+            'findservice',
+            'invalidservice',
+            'servicesetservice',
+            'appliedservice',
+            'inheritedservice',
+        ]);
+    }
+
+    protected function canModifyHostViaMonitoringPermissions($hostname)
+    {
+        if ($this->hasPermission('director/monitoring/hosts')) {
+            $monitoring = new Monitoring();
+            return $monitoring->authCanEditHost($this->Auth(), $hostname);
         }
+
+        return false;
     }
 
     /**
@@ -62,6 +107,7 @@ class HostController extends ObjectController
         $this->addTitle($this->translate('Add Service to %s'), $host->getObjectName());
         $this->content()->add(
             IcingaAddServiceForm::load()
+                ->setBranch($this->getBranch())
                 ->setHost($host)
                 ->setDb($this->db())
                 ->handleRequest()
@@ -73,6 +119,10 @@ class HostController extends ObjectController
         $host = $this->getHostObject();
         $this->addServicesHeader();
         $this->addTitle($this->translate('Add Service Set to %s'), $host->getObjectName());
+        if ($this->showNotInBranch($this->translate('Creating Service Sets'))) {
+            return;
+        }
+
         $this->content()->add(
             IcingaServiceSetForm::load()
                 ->setHost($host)
@@ -100,15 +150,12 @@ class HostController extends ObjectController
         ));
     }
 
-    /**
-     * @throws \Icinga\Exception\NotFoundError
-     */
     public function findserviceAction()
     {
         $host = $this->getHostObject();
-        $redirector = new HostServiceRedirector($host, $this->getAuth());
         $this->redirectNow(
-            $redirector->getRedirectionUrl($this->params->get('service'))
+            (new ServiceFinder($host, $this->getAuth()))
+                ->getRedirectionUrl($this->params->get('service'))
         );
     }
 
@@ -157,12 +204,16 @@ class HostController extends ObjectController
     public function servicesAction()
     {
         $this->addServicesHeader();
-        $db = $this->db();
         $host = $this->getHostObject();
         $this->addTitle($this->translate('Services: %s'), $host->getObjectName());
+        $branch = $this->getBranch();
+        $hostHasBeenCreatedInBranch = $branch->isBranch() && $host->get('id');
         $content = $this->content();
-        $table = IcingaHostServiceTable::load($host)
+        $table = (new ObjectsTableService($this->db()))->setAuth($this->Auth())->setHost($host)
             ->setTitle($this->translate('Individual Service objects'));
+        if ($branch->isBranch()) {
+            $table->setBranchUuid($branch->getUuid());
+        }
 
         if (count($table)) {
             $content->add($table);
@@ -172,18 +223,23 @@ class HostController extends ObjectController
         $parents = IcingaTemplateRepository::instanceByObject($this->object)
             ->getTemplatesFor($this->object, true);
         foreach ($parents as $parent) {
-            $table = IcingaHostServiceTable::load($parent)->setInheritedBy($host);
+            $table = (new ObjectsTableService($this->db()))
+                ->setAuth($this->Auth())
+                ->setHost($parent)
+                ->setInheritedBy($host);
             if (count($table)) {
                 $content->add(
                     $table->setTitle(sprintf(
-                        'Inherited from %s',
+                        $this->translate('Inherited from %s'),
                         $parent->getObjectName()
                     ))
                 );
             }
         }
 
-        $this->addHostServiceSetTables($host);
+        if (! $hostHasBeenCreatedInBranch) {
+            $this->addHostServiceSetTables($host);
+        }
         foreach ($parents as $parent) {
             $this->addHostServiceSetTables($parent, $host);
         }
@@ -227,7 +283,10 @@ class HostController extends ObjectController
         $this->addSingleTab($this->translate('Configuration (read-only)'));
         $this->addTitle($this->translate('Services on %s'), $host->getObjectName());
         $content = $this->content();
-        $table = IcingaHostServiceTable::load($host)
+
+        $table = (new ObjectsTableService($db))
+            ->setAuth($this->Auth())
+            ->setHost($host)
             ->setReadonly()
             ->highlightService($service)
             ->setTitle($this->translate('Individual Service objects'));
@@ -240,8 +299,9 @@ class HostController extends ObjectController
         $parents = IcingaTemplateRepository::instanceByObject($this->object)
             ->getTemplatesFor($this->object, true);
         foreach ($parents as $parent) {
-            $table = IcingaHostServiceTable::load($parent)
+            $table = (new ObjectsTableService($db))
                 ->setReadonly()
+                ->setHost($parent)
                 ->highlightService($service)
                 ->setInheritedBy($host);
             if (count($table)) {
@@ -293,6 +353,9 @@ class HostController extends ObjectController
         if ($affectedHost === null) {
             $affectedHost = $host;
         }
+        if ($host->get('id') === null) {
+            return;
+        }
 
         $query = $db->getDbAdapter()->select()
             ->from(
@@ -556,7 +619,7 @@ class HostController extends ObjectController
      */
     protected function getHostObject()
     {
-        /** @var IcingaHost $this->object */
+        assert($this->object instanceof IcingaHost);
         return $this->object;
     }
 }
