@@ -2,11 +2,16 @@
 
 namespace Icinga\Module\Director\Controllers;
 
+use Exception;
+use gipfl\Web\Widget\Hint;
+use Icinga\Module\Director\Db\Branch\Branch;
 use Icinga\Module\Director\Forms\ImportRowModifierForm;
 use Icinga\Module\Director\Forms\ImportSourceForm;
+use Icinga\Module\Director\Hook\ImportSourceHook;
 use Icinga\Module\Director\Web\ActionBar\AutomationObjectActionBar;
 use Icinga\Module\Director\Web\Controller\ActionController;
 use Icinga\Module\Director\Objects\ImportSource;
+use Icinga\Module\Director\Web\Controller\BranchHelper;
 use Icinga\Module\Director\Web\Form\CloneImportSourceForm;
 use Icinga\Module\Director\Web\Table\ImportrunTable;
 use Icinga\Module\Director\Web\Table\ImportsourceHookTable;
@@ -15,9 +20,13 @@ use Icinga\Module\Director\Web\Tabs\ImportsourceTabs;
 use Icinga\Module\Director\Web\Widget\ImportSourceDetails;
 use InvalidArgumentException;
 use gipfl\IcingaWeb2\Link;
+use ipl\Html\Error;
+use ipl\Html\Html;
 
 class ImportsourceController extends ActionController
 {
+    use BranchHelper;
+
     /** @var ImportSource|null */
     private $importSource;
 
@@ -82,17 +91,29 @@ class ImportsourceController extends ActionController
             $this->translate('Import source: %s'),
             $source->get('source_name')
         )->setAutorefreshInterval(10);
+        $branch = $this->getBranch();
+        if ($this->getBranch()->isBranch()) {
+            $this->content()->add(Hint::info(Html::sprintf($this->translate(
+                'Please note that importing data will take place in your main Branch.'
+                . ' Modifications to Import Sources are not allowed while being in a Configuration Branch.'
+                . ' To get the full functionality, please deactivate %s'
+            ), Branch::requireHook()->linkToBranch($branch, $this->getAuth(), $branch->getName()))));
+        }
         $this->content()->add(new ImportSourceDetails($source));
     }
 
     public function addAction()
     {
-        $this->addTitle($this->translate('Add import source'))
-            ->content()->add(
-                ImportSourceForm::load()->setDb($this->db())
-                    ->setSuccessUrl('director/importsources')
-                    ->handleRequest()
-            );
+        $this->addTitle($this->translate('Add import source'));
+        if ($this->showNotInBranch($this->translate('Creating Import Sources'))) {
+            return;
+        }
+
+        $this->content()->add(
+            ImportSourceForm::load()->setDb($this->db())
+                ->setSuccessUrl('director/importsources')
+                ->handleRequest()
+        );
     }
 
     /**
@@ -102,6 +123,9 @@ class ImportsourceController extends ActionController
     {
         $this->addMainActions();
         $this->activateTabWithPostfix($this->translate('Modify'));
+        if ($this->showNotInBranch($this->translate('Modifying Import Sources'))) {
+            return;
+        }
         $form = ImportSourceForm::load()
             ->setObject($this->getImportSource())
             ->setListUrl('director/importsources')
@@ -121,6 +145,9 @@ class ImportsourceController extends ActionController
     {
         $this->addMainActions();
         $this->activateTabWithPostfix($this->translate('Clone'));
+        if ($this->showNotInBranch($this->translate('Cloning Import Sources'))) {
+            return;
+        }
         $source = $this->getImportSource();
         $this->addTitle('Clone: %s', $source->get('source_name'));
         $form = new CloneImportSourceForm($source);
@@ -142,11 +169,58 @@ class ImportsourceController extends ActionController
             $this->translate('Import source preview: %s'),
             $source->get('source_name')
         );
+        $fetchUrl = clone($this->url());
 
-        $this->actions()->add(Link::create('[..]', '#', null, [
+        $this->actions()->add(Link::create(
+            $this->translate('Download JSON'),
+            $fetchUrl->setPath('director/importsource/fetch'),
+            null,
+            [
+                'target' => '_blank',
+                'class'  => 'icon-download',
+            ]
+        ))->add(Link::create('[..]', '#', null, [
             'onclick' => 'javascript:$("table.raw-data-table").toggleClass("collapsed");'
         ]));
-        (new ImportsourceHookTable())->setImportSource($source)->renderTo($this);
+        try {
+            (new ImportsourceHookTable())->setImportSource($source)->renderTo($this);
+        } catch (Exception $e) {
+            $this->content()->add(Error::show($e));
+        }
+    }
+
+    /**
+     * @throws \Icinga\Exception\ConfigurationError
+     * @throws \Icinga\Exception\NotFoundError
+     * @throws \Icinga\Module\Director\Exception\DuplicateKeyException
+     */
+    public function fetchAction()
+    {
+        $response = $this->getResponse();
+        try {
+            $source = $this->getImportSource();
+            $source->checkForChanges();
+            $hook = ImportSourceHook::forImportSource($source);
+            $data = $hook->fetchData();
+            $source->applyModifiers($data);
+
+            $filename = sprintf(
+                "director-importsource-%d_%s.json",
+                $this->getParam('id'),
+                date('YmdHis')
+            );
+            $response->setHeader('Content-Type', 'application/json', true);
+            $response->setHeader('Content-disposition', "attachment; filename=$filename", true);
+            $response->sendHeaders();
+            $this->sendJson($this->getResponse(), $data);
+        } catch (Exception $e) {
+            $this->sendJsonError($response, $e->getMessage());
+        }
+        // TODO: this is not clean
+        if (\ob_get_level()) {
+            \ob_end_flush();
+        }
+        exit;
     }
 
     /**
@@ -156,9 +230,13 @@ class ImportsourceController extends ActionController
     protected function requireImportSourceAndAddModifierTable()
     {
         $source = $this->getImportSource();
-        PropertymodifierTable::load($source, $this->url())
-            ->handleSortPriorityActions($this->getRequest(), $this->getResponse())
-            ->renderTo($this);
+        $table = PropertymodifierTable::load($source, $this->url());
+        if ($this->getBranch()->isBranch()) {
+            $table->setReadOnly();
+        } else {
+            $table->handleSortPriorityActions($this->getRequest(), $this->getResponse());
+        }
+        $table->renderTo($this);
 
         return $source;
     }
@@ -203,6 +281,10 @@ class ImportsourceController extends ActionController
         )->addBackToModifiersLink($source);
         $this->tabs()->activate('modifier');
 
+        if ($this->showNotInBranch($this->translate('Modifying Import Sources'))) {
+            return;
+        }
+
         $this->content()->prepend(
             ImportRowModifierForm::load()->setDb($this->db())
                 ->setSource($source)
@@ -229,12 +311,15 @@ class ImportsourceController extends ActionController
         )->addBackToModifiersLink($source);
         $source = $this->requireImportSourceAndAddModifierTable();
         $this->tabs()->activate('modifier');
+        if ($this->showNotInBranch($this->translate('Modifying Import Sources'))) {
+            return;
+        }
 
         $listUrl = 'director/importsource/modifier?source_id='
             . (int) $source->get('id');
         $this->content()->prepend(
             ImportRowModifierForm::load()->setDb($this->db())
-                ->loadObject($this->params->getRequired('id'))
+                ->loadObject((int) $this->params->getRequired('id'))
                 ->setListUrl($listUrl)
                 ->setSource($source)
                 ->handleRequest()
