@@ -107,7 +107,7 @@ class IcingaCommandForm extends DirectorObjectForm
                 array('HtmlTag', array('tag' => 'dl')),
                 'Fieldset',
             ),
-            'order' => 80,
+            'order' => self::GROUP_ORDER_CLUSTERING,
             'legend' => $this->translate('Zone settings')
         ));
 
