@@ -713,7 +713,7 @@ class IcingaServiceForm extends DirectorObjectForm
             $table->setBranchUuid($this->branch->getUuid());
         }
         $result = [];
-        foreach ($db->fetchAll($table->getQuery()) as $row) {
+        foreach ($db->fetchAll($table->getQuery()->reset(\Zend_Db_Select::LIMIT_COUNT)) as $row) {
             $result[$row->object_name] = $row->object_name;
         }
 
@@ -783,7 +783,7 @@ class IcingaServiceForm extends DirectorObjectForm
     {
         if ($this->hasBeenSent()) {
             $name = $this->getSentOrObjectValue('object_name');
-            if (!strlen($name)) {
+            if ($name === null || !strlen($name)) {
                 $this->setElementValue('object_name', end($imports));
                 $this->object->set('object_name', end($imports));
             }
