@@ -4,7 +4,7 @@ namespace Icinga\Module\Director\Dashboard\Dashlet;
 
 class NotificationApplyDashlet extends Dashlet
 {
-    protected $icon = 'megaphone';
+    protected $icon = 'bell';
 
     protected $requiredStats = array('notification');
 
@@ -16,9 +16,8 @@ class NotificationApplyDashlet extends Dashlet
     public function getSummary()
     {
         return $this->translate(
-            'Apply notifications with specific properties according to given'
-            . ' rules.' . ' ' . $this->getApplySummaryText('notification')
-        );
+            'Apply notifications with specific properties according to given rules.'
+        )  . ' ' . $this->getApplySummaryText('notification');
     }
 
     public function shouldBeShown()
@@ -33,6 +32,6 @@ class NotificationApplyDashlet extends Dashlet
 
     public function getUrl()
     {
-        return 'director/notifications/apply';
+        return 'director/notifications/applyrules';
     }
 }
