@@ -21,6 +21,7 @@ use Icinga\Module\Director\Objects\SyncRun;
 use Icinga\Exception\IcingaException;
 use Icinga\Module\Director\Repository\IcingaTemplateRepository;
 use InvalidArgumentException;
+use RuntimeException;
 
 class Sync
 {
@@ -106,10 +107,14 @@ class Sync
     {
         $modified = [];
         $objects = $this->prepare();
+        $updateOnly = $this->rule->get('update_policy') === 'update-only';
+        $allowCreate = ! $updateOnly;
         foreach ($objects as $object) {
             if ($object->hasBeenModified()) {
-                $modified[] = $object;
-            } elseif ($object->shouldBeRemoved()) {
+                if ($allowCreate || $object->hasBeenLoadedFromDb()) {
+                    $modified[] = $object;
+                }
+            } elseif (! $updateOnly && $object->shouldBeRemoved()) {
                 $modified[] = $object;
             }
         }
@@ -179,7 +184,7 @@ class Sync
                 $this->hasPropertyDisabled = true;
             }
 
-            if (! strlen($prop->get('filter_expression'))) {
+            if ($prop->get('filter_expression') === null || strlen($prop->get('filter_expression')) === 0) {
                 continue;
             }
 
@@ -629,6 +634,7 @@ class Sync
 
         Benchmark::measure('Modified objects are ready, applying purge strategy');
         $noAction = [];
+        $purgeAction = $this->rule->get('purge_action');
         foreach ($this->rule->purgeStrategy()->listObjectsToPurge() as $key) {
             if (array_key_exists($key, $newObjects)) {
                 // Object has been touched, do not delete
@@ -638,7 +644,18 @@ class Sync
             if (array_key_exists($key, $this->objects)) {
                 $object = $this->objects[$key];
                 if (! $object->hasBeenModified()) {
-                    $object->markForRemoval();
+                    switch ($purgeAction) {
+                        case 'delete':
+                            $object->markForRemoval();
+                            break;
+                        case 'disable':
+                            $object->set('disabled', 'y');
+                            break;
+                        default:
+                            throw new RuntimeException(
+                                "Unsupported purge action: '$purgeAction'"
+                            );
+                    }
                 }
             }
         }
@@ -691,6 +708,7 @@ class Sync
                 break;
 
             case 'merge':
+            case 'update-only':
                 // TODO: re-evaluate merge settings. vars.x instead of
                 //       just "vars" might suffice.
                 $this->objects[$key]->merge($object, $this->replaceVars);
@@ -738,6 +756,8 @@ class Sync
         $dba->beginTransaction();
 
         $object = null;
+        $updateOnly = $this->rule->get('update_policy') === 'update-only';
+        $allowCreate = ! $updateOnly;
 
         try {
             $formerActivityChecksum = hex2bin(
@@ -750,7 +770,7 @@ class Sync
             // $failed = 0;
             foreach ($objects as $object) {
                 $this->setResolver($object);
-                if ($object->shouldBeRemoved()) {
+                if (! $updateOnly && $object->shouldBeRemoved()) {
                     $object->delete();
                     $deleted++;
                     continue;
@@ -758,11 +778,11 @@ class Sync
 
                 if ($object->hasBeenModified()) {
                     $existing = $object->hasBeenLoadedFromDb();
-                    $object->store($db);
-
                     if ($existing) {
+                        $object->store($db);
                         $modified++;
-                    } else {
+                    } elseif ($allowCreate) {
+                        $object->store($db);
                         $created++;
                     }
                 }
