@@ -5,8 +5,11 @@ namespace Icinga\Module\Director\Objects;
 use Icinga\Module\Director\Core\Json;
 use Icinga\Module\Director\Data\Db\DbObjectWithSettings;
 use Icinga\Module\Director\Db;
+use Icinga\Module\Director\DirectorObject\Automation\CompareBasketObject;
 use Icinga\Module\Director\Exception\DuplicateKeyException;
+use Icinga\Module\Director\Forms\IcingaServiceForm;
 use Icinga\Module\Director\Hook\DataTypeHook;
+use Icinga\Module\Director\Resolver\OverriddenVarsResolver;
 use Icinga\Module\Director\Web\Form\DirectorObjectForm;
 use InvalidArgumentException;
 use Zend_Form_Element as ZfElement;
@@ -19,19 +22,27 @@ class DirectorDatafield extends DbObjectWithSettings
 
     protected $autoincKeyName = 'id';
 
-    protected $defaultProperties = array(
+    protected $defaultProperties = [
         'id'            => null,
+        'category_id'   => null,
         'varname'       => null,
         'caption'       => null,
         'description'   => null,
         'datatype'      => null,
         'format'        => null,
-    );
+    ];
+
+    protected $relations = [
+        'category'      => 'DirectorDatafieldCategory'
+    ];
 
     protected $settingsTable = 'director_datafield_setting';
 
     protected $settingsRemoteId = 'datafield_id';
 
+    /** @var DirectorDatafieldCategory|null */
+    private $category;
+
     private $object;
 
     public static function fromDbRow($row, Db $connection)
@@ -53,6 +64,59 @@ class DirectorDatafield extends DbObjectWithSettings
         return $obj;
     }
 
+    public function hasCategory()
+    {
+        return $this->category !== null || $this->get('category_id') !== null;
+    }
+
+    /**
+     * @return DirectorDatafieldCategory|null
+     * @throws \Icinga\Exception\NotFoundError
+     */
+    public function getCategory()
+    {
+        if ($this->category) {
+            return $this->category;
+        } elseif ($id = $this->get('category_id')) {
+            return DirectorDatafieldCategory::loadWithAutoIncId($id, $this->getConnection());
+        } else {
+            return null;
+        }
+    }
+
+    public function getCategoryName()
+    {
+        $category = $this->getCategory();
+        if ($category === null) {
+            return null;
+        } else {
+            return $category->get('category_name');
+        }
+    }
+
+    public function setCategory($category)
+    {
+        if ($category === null) {
+            $this->category = null;
+            $this->set('category_id', null);
+        } elseif ($category instanceof DirectorDatafieldCategory) {
+            if ($category->hasBeenLoadedFromDb()) {
+                $this->set('category_id', $category->get('id'));
+            }
+            $this->category = $category;
+        } else {
+            if (DirectorDatafieldCategory::exists($category, $this->getConnection())) {
+                $this->setCategory(DirectorDatafieldCategory::load($category, $this->getConnection()));
+            } else {
+                $this->setCategory(DirectorDatafieldCategory::create([
+                    'category_name' => $category
+                ], $this->getConnection()));
+            }
+        }
+
+        return $this;
+    }
+
     /**
      * @return object
      * @throws \Icinga\Exception\NotFoundError
@@ -71,6 +135,10 @@ class DirectorDatafield extends DbObjectWithSettings
             )->get('list_name');
             unset($plain->settings->datalist_id);
         }
+        if (property_exists($plain, 'category_id')) {
+            $plain->category = $this->getCategoryName();
+            unset($plain->category_id);
+        }
 
         return $plain;
     }
@@ -102,15 +170,13 @@ class DirectorDatafield extends DbObjectWithSettings
             $list = null;
         }
 
-        $encoded = Json::encode($properties);
-        if ($id) {
-            if (static::exists($id, $db)) {
-                $existing = static::loadWithAutoIncId($id, $db);
-                $existingProperties = (array) $existing->export();
-                unset($existingProperties['originalId']);
-                if ($encoded === Json::encode($existingProperties)) {
-                    return $existing;
-                }
+        $compare = Json::decode(Json::encode($properties));
+        if ($id && static::exists($id, $db)) {
+            $existing = static::loadWithAutoIncId($id, $db);
+            $existingProperties = (array) $existing->export();
+            unset($existingProperties['originalId']);
+            if (CompareBasketObject::equals((object) $compare, (object) $existingProperties)) {
+                return $existing;
             }
         }
 
@@ -128,7 +194,8 @@ class DirectorDatafield extends DbObjectWithSettings
         foreach ($candidates as $candidate) {
             $export = $candidate->export();
             unset($export->originalId);
-            if (Json::encode($export) === $encoded) {
+            CompareBasketObject::normalize($export);
+            if (CompareBasketObject::equals($export, $compare)) {
                 return $candidate;
             }
         }
@@ -136,6 +203,16 @@ class DirectorDatafield extends DbObjectWithSettings
         return static::create($properties, $db);
     }
 
+    protected function beforeStore()
+    {
+        if ($this->category) {
+            if (!$this->category->hasBeenLoadedFromDb()) {
+                throw new \RuntimeException('Trying to store a datafield with an unstored Category');
+            }
+            $this->set('category_id', $this->category->get('id'));
+        }
+    }
+
     protected function setObject(IcingaObject $object)
     {
         $this->object = $object;
@@ -188,57 +265,80 @@ class DirectorDatafield extends DbObjectWithSettings
     protected function applyObjectData(ZfElement $el, DirectorObjectForm $form)
     {
         $object = $form->getObject();
-        if ($object instanceof IcingaObject) {
-            if ($object->isTemplate()) {
-                $el->setRequired(false);
-            }
+        if (! ($object instanceof IcingaObject)) {
+            return;
+        }
+        if ($object->isTemplate()) {
+            $el->setRequired(false);
+        }
 
-            $varname = $this->get('varname');
+        $varName = $this->get('varname');
+        $inherited = $origin = null;
 
-            $inherited = $object->getInheritedVar($varname);
-
-            if (null !== $inherited) {
-                $form->setInheritedValue(
-                    $el,
-                    $inherited,
-                    $object->getOriginForVar($varname)
-                );
-            } elseif ($object->hasRelation('check_command')) {
-                // TODO: Move all of this elsewhere and test it
-                try {
-                    /** @var IcingaCommand $command */
-                    $command = $object->getResolvedRelated('check_command');
-                    if ($command === null) {
-                        return;
-                    }
-                    $inherited = $command->vars()->get($varname);
-                    $inheritedFrom = null;
-
-                    if ($inherited !== null) {
-                        $inherited = $inherited->getValue();
-                    }
-
-                    if ($inherited === null) {
-                        $inherited = $command->getResolvedVar($varname);
-                        if ($inherited === null) {
-                            $inheritedFrom = $command->getOriginForVar($varname);
-                        }
-                    } else {
-                        $inheritedFrom = $command->getObjectName();
-                    }
-
-                    $inherited = $command->getResolvedVar($varname);
-                    if (null !== $inherited) {
-                        $form->setInheritedValue(
-                            $el,
-                            $inherited,
-                            $inheritedFrom
-                        );
-                    }
-                } catch (\Exception $e) {
-                    // Ignore failures
+        if ($form instanceof IcingaServiceForm && $form->providesOverrides()) {
+            $resolver = new OverriddenVarsResolver($form->getDb());
+            $vars = $resolver->fetchForServiceName($form->getHost(), $object->getObjectName());
+            foreach ($vars as $host => $values) {
+                if (\property_exists($values, $varName)) {
+                    $inherited = $values->$varName;
+                    $origin = $host;
                 }
             }
         }
+
+        if ($inherited === null) {
+            $inherited = $object->getInheritedVar($varName);
+            if (null !== $inherited) {
+                $origin = $object->getOriginForVar($varName);
+            }
+        }
+
+        if ($inherited === null) {
+            $cmd = $this->eventuallyGetResolvedCommandVar($object, $varName);
+            if ($cmd !== null) {
+                list($inherited, $origin) = $cmd;
+            }
+        }
+
+        if ($inherited !== null) {
+            $form->setInheritedValue($el, $inherited, $origin);
+        }
+    }
+
+    protected function eventuallyGetResolvedCommandVar(IcingaObject $object, $varName)
+    {
+        if (! $object->hasRelation('check_command')) {
+            return null;
+        }
+
+        // TODO: Move all of this elsewhere and test it
+        try {
+            /** @var IcingaCommand $command */
+            $command = $object->getResolvedRelated('check_command');
+            if ($command === null) {
+                return null;
+            }
+            $inherited = $command->vars()->get($varName);
+            $inheritedFrom = null;
+
+            if ($inherited !== null) {
+                $inherited = $inherited->getValue();
+            }
+
+            if ($inherited === null) {
+                $inherited = $command->getResolvedVar($varName);
+                if ($inherited === null) {
+                    $inheritedFrom = $command->getOriginForVar($varName);
+                }
+            } else {
+                $inheritedFrom = $command->getObjectName();
+            }
+
+            $inherited = $command->getResolvedVar($varName);
+
+            return [$inherited, $inheritedFrom];
+        } catch (\Exception $e) {
+            return null;
+        }
     }
 }
