@@ -9,6 +9,7 @@ use Icinga\Module\Director\Db;
 use Icinga\Module\Director\DirectorObject\Automation\ExportInterface;
 use Icinga\Module\Director\Exception\DuplicateKeyException;
 use Icinga\Module\Director\IcingaConfig\IcingaConfig;
+use Icinga\Module\Director\IcingaConfig\IcingaConfigHelper as c;
 use Icinga\Module\Director\IcingaConfig\IcingaLegacyConfigHelper as c1;
 use Icinga\Module\Director\Objects\Extension\FlappingSupport;
 use InvalidArgumentException;
@@ -22,6 +23,7 @@ class IcingaHost extends IcingaObject implements ExportInterface
 
     protected $defaultProperties = array(
         'id'                      => null,
+        'uuid'                    => null,
         'object_name'             => null,
         'object_type'             => null,
         'disabled'                => 'n',
@@ -54,6 +56,7 @@ class IcingaHost extends IcingaObject implements ExportInterface
         'has_agent'               => null,
         'master_should_connect'   => null,
         'accept_config'           => null,
+        'custom_endpoint_name'    => null,
         'api_key'                 => null,
         'template_choice_id'      => null,
     );
@@ -101,6 +104,8 @@ class IcingaHost extends IcingaObject implements ExportInterface
     /** @var HostGroupMembershipResolver */
     protected $hostgroupMembershipResolver;
 
+    protected $uuidColumn = 'uuid';
+
     public static function enumProperties(
         DbConnection $connection = null,
         $prefix = '',
@@ -130,6 +135,8 @@ class IcingaHost extends IcingaObject implements ExportInterface
 
             $hostProperties[$prefix . $prop] = $prop;
         }
+        unset($hostProperties[$prefix . 'uuid']);
+        unset($hostProperties[$prefix . 'custom_endpoint_name']);
 
         $hostVars = array();
 
@@ -212,7 +219,8 @@ class IcingaHost extends IcingaObject implements ExportInterface
             return;
         }
 
-        $name = $this->object_name;
+        $name = $this->getEndpointName();
+
         if (IcingaEndpoint::exists($name, $this->connection)) {
             return;
         }
@@ -246,6 +254,40 @@ class IcingaHost extends IcingaObject implements ExportInterface
         $config->configFile($pre . 'agent_zones')->addObject($zone);
     }
 
+    /**
+    // @codingStandardsIgnoreStart
+     * @param $value
+     * @return string
+     */
+    protected function renderCustom_endpoint_name($value)
+    {
+        // @codingStandardsIgnoreEnd
+        // When feature flag feature_custom_endpoint is enabled, render custom var
+        if ($this->connection->settings()->get('feature_custom_endpoint') === 'y') {
+            return c::renderKeyValue('vars._director_custom_endpoint_name', c::renderString($value));
+        }
+
+        return '';
+    }
+
+    /**
+     * Returns the hostname or custom endpoint name of the Icinga agent
+     *
+     * @return string
+     */
+    public function getEndpointName()
+    {
+        $name = $this->getObjectName();
+
+        if ($this->connection->settings()->get('feature_custom_endpoint') === 'y') {
+            if (($customName = $this->get('custom_endpoint_name')) !== null) {
+                $name = $customName;
+            }
+        }
+
+        return $name;
+    }
+
     public function getAgentListenPort()
     {
         $conn = $this->connection;
