@@ -14,6 +14,7 @@ class IcingaNotification extends IcingaObject implements ExportInterface
 
     protected $defaultProperties = [
         'id'                    => null,
+        'uuid'                  => null,
         'object_name'           => null,
         'object_type'           => null,
         'disabled'              => 'n',
@@ -31,6 +32,8 @@ class IcingaNotification extends IcingaObject implements ExportInterface
         'assign_filter'         => null,
     ];
 
+    protected $uuidColumn = 'uuid';
+
     protected $supportsCustomVars = true;
 
     protected $supportsFields = true;
@@ -69,52 +72,23 @@ class IcingaNotification extends IcingaObject implements ExportInterface
     }
 
     /**
-     * We have distinct properties in the db
-     *
-     * ...but render times only once
-     *
-     * And we skip warnings about underscores in method names:
      * @codingStandardsIgnoreStart
-     *
      * @return string
      */
     protected function renderTimes_begin()
     {
         // @codingStandardsIgnoreEnd
-        $times = (object) [
-            'begin' => c::renderInterval($this->times_begin)
-        ];
-
-        if ($this->get('times_end') !== null) {
-            $times->end = c::renderInterval($this->get('times_end'));
-        }
-
-        return c::renderKeyValue('times', c::renderDictionary($times));
+        return c::renderKeyValue('times.begin', c::renderInterval($this->get('times_begin')));
     }
 
     /**
-     * We have distinct properties in the db
-     *
-     * ...but render times only once
-     *
-     * And we skip warnings about underscores in method names:
      * @codingStandardsIgnoreStart
-     *
      * @return string
      */
     protected function renderTimes_end()
     {
         // @codingStandardsIgnoreEnd
-
-        if ($this->get('times_begin') !== null) {
-            return '';
-        }
-
-        $times = (object) [
-            'end' => c::renderInterval($this->get('times_end'))
-        ];
-
-        return c::renderKeyValue('times', c::renderDictionary($times));
+        return c::renderKeyValue('times.end', c::renderInterval($this->get('times_end')));
     }
 
     public function getUniqueIdentifier()
