@@ -2,6 +2,7 @@
 
 namespace Icinga\Module\Director\Web\Controller;
 
+use gipfl\Translation\StaticTranslator;
 use Icinga\Application\Benchmark;
 use Icinga\Data\Paginatable;
 use Icinga\Exception\NotFoundError;
@@ -17,7 +18,6 @@ use Icinga\Web\UrlParams;
 use InvalidArgumentException;
 use gipfl\IcingaWeb2\Translator;
 use gipfl\IcingaWeb2\Link;
-use gipfl\Translation\TranslationHelper;
 use gipfl\IcingaWeb2\Widget\ControlsAndContent;
 use gipfl\IcingaWeb2\Controller\Extension\ControlsAndContentHelper;
 use gipfl\IcingaWeb2\Zf1\SimpleViewRenderer;
@@ -60,7 +60,7 @@ abstract class ActionController extends Controller implements ControlsAndContent
 
     protected function initializeTranslator()
     {
-        TranslationHelper::setTranslator(new Translator('director'));
+        StaticTranslator::set(new Translator('director'));
     }
 
     public function getAuth()
@@ -216,7 +216,8 @@ abstract class ActionController extends Controller implements ControlsAndContent
             $viewRenderer = new SimpleViewRenderer();
             $viewRenderer->replaceZendViewRenderer();
             $this->view = $viewRenderer->view;
-            if ($this->getOriginalUrl()->getParam('view') === 'compact') {
+            // Hint -> $this->view->compact is the only way since v2.8.0
+            if ($this->view->compact || $this->getOriginalUrl()->getParam('view') === 'compact') {
                 if ($this->view->controls) {
                     $this->controls()->getAttributes()->add('style', 'display: none;');
                 }
