@@ -105,15 +105,15 @@ class IcingaServiceSetServiceTable extends ZfQueryBasedTable
     {
         if ($this->readonly) {
             if ($this->highlightedService === $row->service) {
-                return Html::tag('span', ['class' => 'icon-right-big'], $row->service);
-            } else {
-                return $row->service;
+                return Html::tag('span', ['class' => 'ro-service icon-right-big'], $row->service);
             }
+
+            return Html::tag('span', ['class' => 'ro-service'], $row->service);
         }
 
         if ($this->affectedHost) {
             $params = [
-                'name'    => $this->affectedHost->getObjectName(),
+                'uuid'    => $this->affectedHost->getUniqueId()->toString(),
                 'service' => $row->service,
                 'set'     => $row->service_set
             ];
