@@ -8,6 +8,7 @@ use gipfl\IcingaWeb2\Link;
 use gipfl\IcingaWeb2\Table\ZfQueryBasedTable;
 use gipfl\IcingaWeb2\Url;
 use Icinga\Module\Director\Restriction\FilterByNameRestriction;
+use Ramsey\Uuid\Uuid;
 
 class ObjectSetTable extends ZfQueryBasedTable
 {
@@ -15,6 +16,7 @@ class ObjectSetTable extends ZfQueryBasedTable
         'os.object_name',
         'os.description',
         'os.assign_filter',
+        'o.object_name',
     ];
 
     private $type;
@@ -37,17 +39,15 @@ class ObjectSetTable extends ZfQueryBasedTable
 
     public function getColumnsToBeRendered()
     {
-        return ['Name'];
+        return [$this->translate('Name')];
     }
 
     public function renderRow($row)
     {
         $type = $this->getType();
-        if ($row->object_type === 'apply') {
-            $params['id'] = $row->id;
-        } else {
-            $params = array('name' => $row->object_name);
-        }
+        $params = [
+            'uuid' => Uuid::fromBytes(Db\DbUtil::binaryResult($row->uuid))->toString(),
+        ];
 
         $url = Url::fromPath("director/${type}set", $params);
 
@@ -69,6 +69,7 @@ class ObjectSetTable extends ZfQueryBasedTable
 
         $columns = [
             'id'             => 'os.id',
+            'uuid'           => 'os.uuid',
             'object_name'    => 'os.object_name',
             'object_type'    => 'os.object_type',
             'assign_filter'  => 'os.assign_filter',
